/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef DOCUMENT_H_INCLUDED
#define DOCUMENT_H_INCLUDED

#include <c4scripter/c4s-customfunctionlist.h>
#include <c4scripter/c4s-highlighter.h>
#include <c4scripter/c4s-function.h>
#include <c4scripter/c4s-preferences.h>

#include <gtkmm.h>
#include <gtksourceview/gtksourceview.h>

#include "ParamInfo.h"

class Document: public Gtk::ScrolledWindow
{
public:
  typedef sigc::signal<void> SignalTitleChanged;
  typedef sigc::signal<void> SignalPathChanged;

  typedef std::deque<std::string> KeywordList;
 
  Document(const KeywordList& keywords, C4sPreferences* preferences, C4sEngineFunctionList* engine_function_list);
  Document(const std::string& file, const KeywordList& keywords, C4sPreferences* preferences, C4sEngineFunctionList* engine_function_list);
  ~Document();

  void save(const std::string& file);
  const std::string& get_title() const { return m_title; }
  const std::string& get_path() const;
	
  C4sCustomFunctionList* get_function_list() { return m_function_list; }
  const C4sCustomFunctionList* get_function_list() const { return m_function_list; }

  ParamInfo& get_param_info() { return m_param_info; }
  const ParamInfo& get_param_info() const { return m_param_info; }

  Gtk::TextView* get_text_view() { return Glib::wrap(GTK_TEXT_VIEW(m_sourceview)); }
  Glib::RefPtr<Gtk::TextBuffer> get_text_buffer() { return Glib::wrap(gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_sourceview)), true);  }

  void jump_to_function(const C4sCustomFunction* function);

  SignalTitleChanged signal_title_changed() const { return m_signal_title_changed; }
  SignalPathChanged signal_path_changed() const { return m_signal_path_changed; }
private:
  static void on_notify(GObject* object, GParamSpec* pspec, gpointer data);

  void init_preferences();

  void on_color_changed();
  void on_tab_width_changed();
  void on_auto_indentation_changed();
  void on_highlight_bracket_changed();
  void on_font_changed();

  C4sPreferences* m_preferences;

  std::string m_title;
  std::string m_path;

  std::string m_encoding;
  enum { DOS, UNIX, MAC } m_line_ending;

  GtkTextBuffer* m_textbuffer;
  C4sHighlighter* m_highlighter;
  C4sCustomFunctionList* m_function_list;
  ParamInfo m_param_info;
  GtkWidget* m_sourceview;

  SignalTitleChanged m_signal_title_changed;
  SignalPathChanged m_signal_path_changed;
};

#endif // DOCUMENT_H_INCLUDED
