/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FILEHANDLER_H_INCLUDED
#define FILEHANDLER_H_INCLUDED

#include <gtkmm.h>
#include "MainMenu.h"
#include "FileHolder.h"
#include "CloseConfirmation.h"

class FileHandler
{
public:
  FileHandler(MainMenu& menu, FileHolder& file_holder, Gtk::Window& parent);

  // Return false if the user cancelled the operation or an error occured
  bool new_document();
  bool open_document();
  bool open_document_from(const std::string& path);
  bool save_document(Document& document);
  bool save_document_as(Document& document);
  bool save_all_documents();

  // Closes the given document. A close confirmation will be shown to the user
  // if the document has been changed since it was saved
  bool close_document(Document& document);
  bool close_all_documents();

private:
  void on_document_changed(DocumentHolder* holder);
  void on_document_close(DocumentHolder& holder);

  void on_file_new();
  void on_file_open();
  void on_file_save();
  void on_file_save_as();
  void on_file_save_all();
  void on_file_close();

  bool launch_close_confirmation();

  MainMenu& m_menu;
  FileHolder& m_file_holder;
  Gtk::Window& m_parent;

  std::auto_ptr<CloseConfirmationDialog> m_close_confirmation;

  std::string m_last_path;

  std::string select_open_path();
  std::string select_save_path(const Document& doc);
};

#endif // FILEHANDLER_H_INCLUDED
