/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FUNCTIONHOLDER_H_INCLUDED
#define FUNCTIONHOLDER_H_INCLUDED

#include <c4s-customfunctionlist.h>
#include <c4s-enginefunctionlist.h>
#include <c4s-customfunction.h>
#include <c4s-function.h>

#include <gtkmm.h>
#include "ParamInfo.h"

class FunctionHolder: public Gtk::HBox
{
public:
  typedef sigc::signal<void, const C4sCustomFunction*> SignalFunctionSelected;

  class CustomColumns: public Gtk::TreeModel::ColumnRecord
  {
  public:
    CustomColumns();

      Gtk::TreeModelColumn<Glib::RefPtr<const Gdk::Pixbuf> > access;
      Gtk::TreeModelColumn<Glib::ustring> name;
      Gtk::TreeModelColumn<const C4sCustomFunction*> func;
  };

  FunctionHolder(C4sCustomFunctionList* custom_funcs, C4sEngineFunctionList* engine_funcs, const ParamInfo& param_info);
  ~FunctionHolder();

  Glib::RefPtr<Gtk::ListStore> get_list() { return m_custom_funcs; }
  CustomColumns& get_columns() { return m_custom_columns; }

  SignalFunctionSelected signal_function_selected() const { return m_signal_function_selected; }

private:
  static void on_engine_functions_reloaded_static(C4sEngineFunctionList* list, gpointer user_data) { static_cast<FunctionHolder*>(user_data)->on_engine_functions_reloaded(); }
  void on_engine_functions_reloaded();

  static void on_function_added_static(C4sCustomFunctionList* list, C4sCustomFunction* function, gpointer user_data) { static_cast<FunctionHolder*>(user_data)->on_function_added(function); }
  static void on_function_removed_static(C4sCustomFunctionList* list, C4sCustomFunction* function, gpointer user_data) { static_cast<FunctionHolder*>(user_data)->on_function_removed(function); }
  static void on_function_added_static2(C4sCustomFunction* function, gpointer user_data) { static_cast<FunctionHolder*>(user_data)->on_function_added(function); }

  void on_function_added(const C4sCustomFunction* func);
  void on_function_removed(const C4sCustomFunction* func);

  static void on_function_access_changed_static(const C4sCustomFunction* func, C4sCustomFunctionAccess access, gpointer user_data);
  static void on_function_name_changed_static(const C4sCustomFunction* func, const gchar* name, gpointer user_data);

  void on_function_access_changed(const C4sCustomFunction* func);
  void on_function_name_changed(const C4sCustomFunction* func);

  void on_param_info_changed(const C4sFunction* func, unsigned int param);
  void on_param_info_removed();

  void on_custom_function_changed();

  Gtk::TreeIter find_function(const C4sCustomFunction* func);

  C4sCustomFunctionList* m_custom_list;
  C4sEngineFunctionList* m_engine_list;
  const ParamInfo& m_param_info;

  CustomColumns m_custom_columns;
  Glib::RefPtr<Gtk::ListStore> m_custom_funcs;
  Glib::RefPtr<Gtk::TreeModelSort> m_custom_sort;

  Gtk::ComboBox m_custom_box;
  Gtk::ComboBox m_engine_box;

  Gtk::CellRendererPixbuf m_custom_renderer_access;
  Gtk::CellRendererText m_custom_renderer_name;

  Gtk::CellRendererText m_engine_renderer;

  sigc::connection m_conn_custom_changed;
  SignalFunctionSelected m_signal_function_selected;
};

#endif // FUNCTIONHOLDER_H_INCLUDED
