/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PARAMINFO_H_INCLUDED
#define PARAMINFO_H_INCLUDED

#include <c4scripter/c4s-customfunctionlist.h>
#include <c4scripter/c4s-highlighter.h>
#include <c4scripter/c4s-enginefunctionlist.h>
#include <c4scripter/c4s-function.h>

#include <gtkmm.h>

class ParamInfo
{
public:
  typedef sigc::signal<void, const C4sFunction*, unsigned int> SignalFunctionChanged;
  typedef sigc::signal<void> SignalFunctionRemoved;

  ParamInfo(const Glib::RefPtr<Gtk::TextBuffer>& buffer, const C4sHighlighter* highlighter, C4sCustomFunctionList* custom_function_list, C4sEngineFunctionList* engine_function_list);
  ~ParamInfo();

  const C4sFunction* get_current_function() const { return cur_function; }
  unsigned int get_current_parameter() const { return cur_param; }

  SignalFunctionChanged signal_function_changed() const { return m_signal_function_changed; }
  SignalFunctionRemoved signal_function_removed() const { return m_signal_function_removed; }
private:
  static void on_engine_functions_reloaded_static(C4sEngineFunctionList* list, gpointer user_data) { static_cast<ParamInfo*>(user_data)->on_engine_functions_reloaded(); }
  void on_engine_functions_reloaded();

  void on_cursor_moved(const Gtk::TextIter& iter, const Glib::RefPtr<Gtk::TextMark>& mark);

  static void on_highlighted_static(C4sHighlighter* highlighter, const GtkTextIter* from, const GtkTextIter* to, gpointer user_data);
  void on_highlighted(const Gtk::TextIter& from, const Gtk::TextIter& to);

  void update();

  Glib::RefPtr<Gtk::TextBuffer> m_buffer;
  const C4sHighlighter* m_highlighter;
  C4sCustomFunctionList* m_custom_function_list;
  C4sEngineFunctionList* m_engine_function_list;

  const C4sFunction* cur_function;
  unsigned int cur_param;

  SignalFunctionChanged m_signal_function_changed;
  SignalFunctionRemoved m_signal_function_removed;
};

#endif // PARAMINFO_H_INCLUDED
