/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PARAMTIP_H_INCLUDED
#define PARAMTIP_H_INCLUDED

#include <gtkmm.h>
#include <c4scripter/c4s-preferences.h>
#include <c4scripter/c4s-customfunction.h>

#include "FileHolder.h"

class ParamTip: public sigc::trackable
{
public:
  ParamTip(FileHolder& file_holder, C4sPreferences* preferences, Gtk::Window& parent);

private:
  static void on_notify(GObject* object, GParamSpec* pspec, gpointer data);
  void install_event_handler();

  void on_document_changed(DocumentHolder* doc_holder);
  void on_function_changed();
  void on_function_removed();

  bool on_frame(GdkEvent* event);
  bool on_focus_out(GdkEventFocus* event);
  bool on_focus_in(GdkEventFocus* event);
  bool on_window_state(GdkEventWindowState* event);
  void on_paramtips_changed();

  void on_hscrolled();
  void on_vscrolled();

#ifdef _WIN32
  static GdkFilterReturn on_filter(GdkXEvent* xevent, GdkEvent* event, gpointer userdata);
#endif

  bool modify_state(unsigned int add, unsigned int remove);
  void refresh_tip(int offset_x = 0, int offset_y = 0);
  void show_tip(const C4sFunction* function, unsigned int parameter, int offset_x = 0, int offset_y = 0);

  FileHolder& m_file_holder;
  C4sPreferences* m_preferences;
  Gtk::Window& m_parent;

  Document* m_current_document;

  Gtk::Window m_tip_wnd;
  Gtk::EventBox m_eb;
  Gtk::Label m_text;

  unsigned int m_state;

  int m_scroll_x;
  int m_scroll_y;

  sigc::connection m_conn_changed;
  sigc::connection m_conn_removed;
  sigc::connection m_conn_hscrolled;
  sigc::connection m_conn_vscrolled;
};

#endif // PARAMTIP_H_INCLUDED
