/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-functionquerydialog.h>

#include <gtk/gtkimage.h>
#include <gtk/gtklabel.h>
#include <gtk/gtktable.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkstock.h>

typedef struct _C4sFunctionQueryDialogPrivate C4sFunctionQueryDialogPrivate;
struct _C4sFunctionQueryDialogPrivate {
  C4sPreferences* preferences;

  C4sFunctionQueryWidget* query_widget;
};

#define C4S_FUNCTION_QUERY_DIALOG_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_FUNCTION_QUERY_DIALOG, \
    C4sFunctionQueryDialogPrivate \
  ))

enum {
  PROP_0,

  /* construct only */
  PROP_PREFERENCES
};

static GtkDialogClass* parent_class = NULL;

static void
c4s_function_query_dialog_update_cb(G_GNUC_UNUSED
                                    C4sFunctionQueryWidget* widget,
                                    C4sFunctionQueryAction action,
                                    const gchar* chm_file,
                                    gpointer user_data)
{
  C4sFunctionQueryDialog* dialog;
  C4sFunctionQueryDialogPrivate* priv;

  dialog = C4S_FUNCTION_QUERY_DIALOG(user_data);
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);

  c4s_preferences_set_function_query_action(priv->preferences, action);
  if(chm_file != NULL)
    c4s_preferences_set_chm_file_path(priv->preferences, chm_file);
  
  gtk_dialog_set_response_sensitive(
    GTK_DIALOG(dialog),
    GTK_RESPONSE_ACCEPT,
    action == C4S_FUNCTION_QUERY_ACTION_ONLINE || chm_file != NULL
  );
}

static void
c4s_function_query_dialog_init(C4sFunctionQueryDialog* dialog)
{
  gint size;
  PangoFontDescription* font_desc;

  GtkWidget* table;

  GtkWidget* vbox_label;

  GtkWidget* label_primary;
  GtkWidget* label_secondary;

  GtkWidget* image;

  C4sFunctionQueryDialogPrivate* priv;

  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);
  priv->preferences = NULL;

  image = gtk_image_new_from_stock(
    GTK_STOCK_DIALOG_QUESTION,
    GTK_ICON_SIZE_DIALOG
  );

  gtk_misc_set_alignment(GTK_MISC(image), 0.5, 0.0);
  gtk_widget_show(image);

  label_primary = gtk_label_new(
    "C4Script function database needs to be created"
  );

  size = pango_font_description_get_size(label_primary->style->font_desc);
  font_desc = pango_font_description_new();
  pango_font_description_set_weight(font_desc, PANGO_WEIGHT_BOLD);
  pango_font_description_set_size(font_desc, size * PANGO_SCALE_LARGE);
  gtk_widget_modify_font(label_primary, font_desc);
  pango_font_description_free(font_desc);

  gtk_label_set_line_wrap(GTK_LABEL(label_primary), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label_primary), 0.0, 0.0);
  gtk_widget_show(label_primary);

  label_secondary = gtk_label_new(
    "C4Scripter needs to get C4Script function definitions because the "
    "local function database is either not present, damaged or outdated. "
    "It can either read them from the documentation CHM file or get them "
    "online from http://clonk.de."
  );

  gtk_label_set_line_wrap(GTK_LABEL(label_secondary), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label_secondary), 0.0, 0.0);
  gtk_widget_show(label_secondary);
  
  vbox_label = gtk_vbox_new(FALSE, 12);
  gtk_box_pack_start(GTK_BOX(vbox_label), label_primary, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox_label), label_secondary, FALSE, FALSE, 0);
  gtk_widget_show(vbox_label);

  priv->query_widget = c4s_function_query_widget_new(GTK_WINDOW(dialog));
  gtk_widget_show(GTK_WIDGET(priv->query_widget));

  table = gtk_table_new(2, 2, FALSE);
  gtk_table_attach(
    GTK_TABLE(table),
    image,
    0, 1, 0, 2,
    GTK_SHRINK | GTK_FILL,
    GTK_SHRINK | GTK_FILL,
    0, 0
  );

  gtk_table_attach(
    GTK_TABLE(table),
    vbox_label,
    1, 2, 0, 1,
    GTK_EXPAND | GTK_FILL,
    GTK_SHRINK | GTK_FILL,
    0, 0
  );

  gtk_table_attach(
    GTK_TABLE(table),
    GTK_WIDGET(priv->query_widget),
    1, 2, 1, 2,
    GTK_EXPAND | GTK_FILL,
    GTK_SHRINK | GTK_FILL,
    0, 0
  );

  gtk_table_set_row_spacings(GTK_TABLE(table), 12);
  gtk_table_set_col_spacings(GTK_TABLE(table), 12);
  gtk_widget_show(table);

  gtk_box_pack_start(
    GTK_BOX(GTK_DIALOG(dialog)->vbox),
    table,
    FALSE,
    FALSE,
    0
  );

  gtk_dialog_add_button(
    GTK_DIALOG(dialog),
    GTK_STOCK_CANCEL,
    GTK_RESPONSE_CANCEL
  );

  gtk_dialog_add_button(
    GTK_DIALOG(dialog),
    GTK_STOCK_OK,
    GTK_RESPONSE_ACCEPT
  );

  gtk_dialog_set_default_response(
    GTK_DIALOG(dialog),
    GTK_RESPONSE_ACCEPT
  );

  g_signal_connect_after(
    G_OBJECT(priv->query_widget),
    "update",
    G_CALLBACK(c4s_function_query_dialog_update_cb),
    dialog
  );

  gtk_window_set_title(GTK_WINDOW(dialog), "Function database");
  gtk_container_set_border_width(GTK_CONTAINER(table), 12);
}

static void 
c4s_function_query_dialog_finalize(GObject* object)
{
  C4sFunctionQueryDialog* function_query_dialog;
  C4sFunctionQueryDialogPrivate* priv;

  function_query_dialog = C4S_FUNCTION_QUERY_DIALOG(object);
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(function_query_dialog);

  if(priv->preferences != NULL) g_object_unref(G_OBJECT(priv->preferences));

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_function_query_dialog_set_property(GObject* object,
                                       guint prop_id,
                                       const GValue* value,
                                       GParamSpec* pspec)
{
  C4sFunctionQueryDialog* dialog;
  C4sFunctionQueryDialogPrivate* priv;
  const gchar* chm_file;

  g_return_if_fail(C4S_IS_FUNCTION_QUERY_DIALOG(object));

  dialog = C4S_FUNCTION_QUERY_DIALOG(object);
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    if(priv->preferences != NULL)
      g_object_unref(G_OBJECT(priv->preferences));

    /* TODO: React on changes to action and chm path */
    priv->preferences = C4S_PREFERENCES(g_value_dup_object(value));
    chm_file = c4s_preferences_get_chm_file_path(priv->preferences);

    c4s_function_query_widget_set_action(
      priv->query_widget,
      c4s_preferences_get_function_query_action(priv->preferences)
    );

    if(*chm_file != '\0')
      c4s_function_query_widget_set_chm(priv->query_widget, chm_file);

    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void
c4s_function_query_dialog_get_property(GObject* object,
                                       guint prop_id,
                                       G_GNUC_UNUSED GValue* value,
                                       GParamSpec* pspec)
{
  C4sFunctionQueryDialog* dialog;
  C4sFunctionQueryDialogPrivate* priv;

  g_return_if_fail(C4S_IS_FUNCTION_QUERY_DIALOG(object));

  dialog = C4S_FUNCTION_QUERY_DIALOG(object);
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);

  switch(prop_id)
  {
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
}

static void
c4s_function_query_dialog_class_init(C4sFunctionQueryDialogClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = GTK_DIALOG_CLASS(g_type_class_peek_parent(klass));

  g_type_class_add_private(klass, sizeof(C4sFunctionQueryDialogPrivate));

  object_class->finalize = c4s_function_query_dialog_finalize;
  object_class->set_property = c4s_function_query_dialog_set_property;
  object_class->get_property = c4s_function_query_dialog_get_property;

  g_object_class_install_property(
    object_class,
    PROP_PREFERENCES,
    g_param_spec_object(
      "preferences",
      "Preferences",
      "Preferences to read CHM file path from",
      C4S_TYPE_PREFERENCES,
      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY
    )
  );
}

GType
c4s_function_query_dialog_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sFunctionQueryDialogClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_function_query_dialog_class_init,
      NULL,
      NULL,
      sizeof(C4sFunctionQueryDialog),
      0,
      (GInstanceInitFunc)c4s_function_query_dialog_init
    };

    our_type = g_type_register_static(
      GTK_TYPE_DIALOG,
      "C4sFunctionQueryDialog",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sFunctionQueryDialog*
c4s_function_query_dialog_new(GtkWindow* parent,
                              GtkDialogFlags flags,
                              C4sPreferences* preferences)
{
  GObject* object;

  object = g_object_new(
    C4S_TYPE_FUNCTION_QUERY_DIALOG,
    "preferences", preferences,
    NULL
  );

  if(flags & GTK_DIALOG_MODAL)
    gtk_window_set_modal(GTK_WINDOW(object), TRUE);

  if(flags & GTK_DIALOG_DESTROY_WITH_PARENT)
    gtk_window_set_destroy_with_parent(GTK_WINDOW(object), TRUE);

  if(flags & GTK_DIALOG_NO_SEPARATOR)
    gtk_dialog_set_has_separator(GTK_DIALOG(object), FALSE);

  gtk_window_set_transient_for(GTK_WINDOW(object), parent);

  return C4S_FUNCTION_QUERY_DIALOG(object);
}

C4sFunctionQueryAction
c4s_function_query_dialog_get_action(const C4sFunctionQueryDialog* dialog)
{
  C4sFunctionQueryDialogPrivate* priv;
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);
  
  return c4s_function_query_widget_get_action(priv->query_widget);
}

const gchar*
c4s_function_query_dialog_get_chm(const C4sFunctionQueryDialog* dialog)
{
  C4sFunctionQueryDialogPrivate* priv;
  priv = C4S_FUNCTION_QUERY_DIALOG_PRIVATE(dialog);

  /* Read from preferences rather than FunctionQueryWidget, so
   * the caller does not have to free it */
  return c4s_preferences_get_chm_file_path(priv->preferences);
}
