/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_PREFERENCES_H__
#define __C4S_PREFERENCES_H__

#include <c4scripter/c4s-functionquerywidget.h>

#include <gdk/gdkcolor.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_PREFERENCES			(c4s_preferences_get_type())
#define C4S_PREFERENCES(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_PREFERENCES, C4sPreferences))
#define C4S_PREFERENCES_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_PREFERENCES, C4sPreferencesClass))
#define C4S_IS_PREFERENCES(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_PREFERENCES))
#define C4S_IS_PREFERENCES_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_PREFERENCES))
#define C4S_PREFERENCES_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_PREFERENCES, C4sPreferencesClass))

typedef struct _C4sStorageBackend C4sStorageBackend;

typedef struct _C4sStorageBackendFuncs C4sStorageBackendFuncs;
struct _C4sStorageBackendFuncs {
  C4sStorageBackend*(*init)(GError**);
  void(*deinit)(C4sStorageBackend*);

  gint(*read_int)(C4sStorageBackend*, const gchar*, const gchar*, gint);
  gboolean(*read_boolean)(C4sStorageBackend*, const gchar*, const gchar*, gboolean);
  gchar*(*read_string)(C4sStorageBackend*, const gchar*, const gchar*, const gchar*);

  void(*write_int)(C4sStorageBackend*, const gchar*, const gchar*, gint);
  void(*write_boolean)(C4sStorageBackend*, const gchar*, const gchar*, gboolean);
  void(*write_string)(C4sStorageBackend*, const gchar*, const gchar*, const gchar*);
};

typedef struct _C4sPreferencesClass C4sPreferencesClass;
typedef struct _C4sPreferences C4sPreferences;

struct _C4sPreferencesClass
{
  GObjectClass parent_class;
};

struct _C4sPreferences
{
  GObject parent_instance;
};

GType
c4s_preferences_get_type(void) G_GNUC_CONST;

C4sPreferences*
c4s_preferences_new(const C4sStorageBackendFuncs* backend);

void
c4s_preferences_save(const C4sPreferences* preferences);

/* Shortcuts for accessing properties */
guint
c4s_preferences_get_tab_width(const C4sPreferences* preferences);

gboolean
c4s_preferences_get_auto_indentation(const C4sPreferences* preferences);

gboolean
c4s_preferences_get_highlight_brackets(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_text(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_string(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_comment(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_keyword(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_function(const C4sPreferences* preferences);

const GdkColor*
c4s_preferences_get_color_number(const C4sPreferences* preferences);

const PangoFontDescription*
c4s_preferences_get_font(const C4sPreferences* preferences);

gboolean
c4s_preferences_get_view_tooltips(const C4sPreferences* preferences);

gboolean
c4s_preferences_get_view_statusbar(const C4sPreferences* preferences);

C4sFunctionQueryAction
c4s_preferences_get_function_query_action(const C4sPreferences* preferences);

const gchar*
c4s_preferences_get_chm_file_path(const C4sPreferences* preferences);

void
c4s_preferences_set_tab_width(C4sPreferences* preferences,
                              guint tab_width);

void
c4s_preferences_set_auto_indentation(C4sPreferences* preferences,
                                     gboolean auto_indentation);

void
c4s_preferences_set_highlight_brackets(C4sPreferences* preferences,
                                       gboolean highlight_brackets);

void
c4s_preferences_set_color_text(C4sPreferences* preferences,
                               const GdkColor* color);

void
c4s_preferences_set_color_string(C4sPreferences* preferences,
                                 const GdkColor* color);
      
void
c4s_preferences_set_color_comment(C4sPreferences* preferences,
                                  const GdkColor* color);

void
c4s_preferences_set_color_keyword(C4sPreferences* preferences,
                                  const GdkColor* color);

void
c4s_preferences_set_color_function(C4sPreferences* preferences,
                                   const GdkColor* color);

void
c4s_preferences_set_color_number(C4sPreferences* preferences,
                                 const GdkColor* color);

void
c4s_preferences_set_font(C4sPreferences* preferences,
                         const PangoFontDescription* font);

void
c4s_preferences_set_view_tooltips(C4sPreferences* preferences,
                                  gboolean view_tooltips);

void
c4s_preferences_set_view_statusbar(C4sPreferences* preferences,
                                   gboolean view_statusbar);

void
c4s_preferences_set_function_query_action(C4sPreferences* preferences,
                                          C4sFunctionQueryAction action);

void
c4s_preferences_set_chm_file_path(C4sPreferences* preferences,
                                  const gchar* file_path);

G_END_DECLS

#endif /* __C4S_PREFERENCES_H__ */
