/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-preferencesdocs.h>
#include <c4scripter/c4s-functionquerywidget.h>

#include <gtk/gtkbutton.h>

#include <string.h>

typedef struct _C4sPreferencesDocsPrivate C4sPreferencesDocsPrivate;
struct _C4sPreferencesDocsPrivate
{
  C4sPreferences* preferences;
  C4sEngineFunctionList* engine_function_list;

  C4sFunctionQueryWidget* query_widget;
  GtkWidget* reload_button;
};

#define C4S_PREFERENCES_DOCS_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_PREFERENCES_DOCS, \
    C4sPreferencesDocsPrivate \
  ))

enum
{
  PROP_0,

  PROP_PREFERENCES,
  PROP_ENGINE_FUNCTION_LIST
};

static C4sPreferencesPageClass* parent_class = NULL;

static void
c4s_preferences_docs_notify_cb(GObject* object,
                                 GParamSpec* pspec,
                                 gpointer user_data);

static void
c4s_preferences_docs_update_cb(G_GNUC_UNUSED C4sFunctionQueryWidget* widget,
                               C4sFunctionQueryAction action,
                               const gchar* chm_path,
                               gpointer user_data)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;

  docs = C4S_PREFERENCES_DOCS(user_data);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);
  
  if(priv->preferences != NULL)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_docs_notify_cb),
      docs
    );
    
    c4s_preferences_set_function_query_action(priv->preferences, action);
    if(chm_path != NULL)
      c4s_preferences_set_chm_file_path(priv->preferences, chm_path);
    
    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_docs_notify_cb),
      docs
    );
  }
}

static void
c4s_preferences_docs_notify_cb(G_GNUC_UNUSED GObject* object,
                               GParamSpec* pspec,
                               gpointer user_data)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;

  docs = C4S_PREFERENCES_DOCS(user_data);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  g_signal_handlers_block_by_func(
    G_OBJECT(priv->query_widget),
    G_CALLBACK(c4s_preferences_docs_update_cb),
    docs
  );

  if(strcmp(pspec->name, "function-query-action") == 0)
  {
    c4s_function_query_widget_set_action(
      priv->query_widget,
      c4s_preferences_get_function_query_action(priv->preferences)
    );
  }
  else if(strcmp(pspec->name, "chm-file-path") == 0)
  {
    c4s_function_query_widget_set_chm(
      priv->query_widget,
      c4s_preferences_get_chm_file_path(priv->preferences)
    );
  }

  g_signal_handlers_unblock_by_func(
    G_OBJECT(priv->query_widget),
    G_CALLBACK(c4s_preferences_docs_update_cb),
    docs
  );
}

static void
c4s_preferences_docs_reload_cb(G_GNUC_UNUSED GtkButton* button,
                               gpointer user_data)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;

  docs = C4S_PREFERENCES_DOCS(user_data);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  if(priv->engine_function_list != NULL)
    c4s_engine_function_list_reload(priv->engine_function_list);
}

static void
c4s_preferences_docs_init(C4sPreferencesDocs* docs)
{
  C4sPreferencesDocsPrivate* priv;
  C4sOptionGroup* group;

  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  priv->preferences = NULL;

  /* TODO: We need some parent here... */
  priv->query_widget = c4s_function_query_widget_new(NULL);
  gtk_widget_show(GTK_WIDGET(priv->query_widget));

  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(docs),
    "Documentation source"
  );

  gtk_box_pack_start(
    GTK_BOX(group->vbox),
    GTK_WIDGET(priv->query_widget),
    FALSE,
    FALSE,
    0
  );
  
  g_signal_connect_after(
    G_OBJECT(priv->query_widget),
    "update",
    G_CALLBACK(c4s_preferences_docs_update_cb),
    docs
  );
  
  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(docs),
    "C4Script functions"
  );
  
  priv->reload_button = gtk_button_new_with_label("Reload functions");
  gtk_widget_show(priv->reload_button);
  
  gtk_box_pack_start(
    GTK_BOX(group->vbox),
    priv->reload_button,
    FALSE,
    FALSE,
    0
  );
  
  g_signal_connect_after(
    G_OBJECT(priv->reload_button),
    "clicked",
    G_CALLBACK(c4s_preferences_docs_reload_cb),
    docs
  );

  gtk_widget_set_sensitive(priv->reload_button, FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(docs), FALSE);
}

static void
c4s_preferences_docs_set_preferences(C4sPreferencesDocs* docs,
                                       C4sPreferences* preferences)
{
  C4sPreferencesDocsPrivate* priv;
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_disconnect_by_func(
      G_OBJECT(preferences),
      G_CALLBACK(c4s_preferences_docs_notify_cb),
      docs
    );

    g_object_unref(G_OBJECT(preferences));

    if(preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(docs), FALSE);
  }

  if(preferences != NULL)
  {
    g_signal_connect(
      G_OBJECT(preferences),
      "notify",
      G_CALLBACK(c4s_preferences_docs_notify_cb),
      docs
    );
    
    c4s_function_query_widget_set_action(
      priv->query_widget,
      c4s_preferences_get_function_query_action(preferences)
    );
    
    c4s_function_query_widget_set_chm(
      priv->query_widget,
      c4s_preferences_get_chm_file_path(preferences)
    );

    g_object_ref(G_OBJECT(preferences));

    if(priv->preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(docs), TRUE);
  }

  priv->preferences = preferences;
}

static void
c4s_preferences_docs_finalize(GObject* object)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;
  
  docs = C4S_PREFERENCES_DOCS(object);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_disconnect_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_docs_notify_cb),
      docs
    );

    g_object_unref(priv->preferences);
  }
  
  if(priv->engine_function_list != NULL)
    g_object_unref(G_OBJECT(priv->engine_function_list));

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_preferences_docs_set_property(GObject* object,
                                    guint prop_id,
                                    const GValue* value,
                                    GParamSpec* pspec)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_DOCS(object));

  docs = C4S_PREFERENCES_DOCS(object);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    c4s_preferences_docs_set_preferences(
      docs,
      C4S_PREFERENCES(g_value_get_object(value))
    );

    break;
  case PROP_ENGINE_FUNCTION_LIST:
    if(priv->engine_function_list != NULL)
      g_object_unref(G_OBJECT(priv->engine_function_list));

    priv->engine_function_list = C4S_ENGINE_FUNCTION_LIST(
      g_value_dup_object(value)
    );
    
    if(priv->engine_function_list != NULL)
      gtk_widget_set_sensitive(priv->reload_button, TRUE);
    else
      gtk_widget_set_sensitive(priv->reload_button, FALSE);

    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_docs_get_property(GObject* object,
                                    guint prop_id,
                                    GValue* value,
                                    GParamSpec* pspec)
{
  C4sPreferencesDocs* docs;
  C4sPreferencesDocsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_DOCS(object));

  docs = C4S_PREFERENCES_DOCS(object);
  priv = C4S_PREFERENCES_DOCS_PRIVATE(docs);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    g_value_set_object(value, priv->preferences);
    break;
  case PROP_ENGINE_FUNCTION_LIST:
    g_value_set_object(value, priv->engine_function_list);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_docs_class_init(C4sPreferencesDocsClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);
  
  g_type_class_add_private(klass, sizeof(C4sPreferencesDocsPrivate));

  object_class->finalize = c4s_preferences_docs_finalize;
  object_class->set_property = c4s_preferences_docs_set_property;
  object_class->get_property = c4s_preferences_docs_get_property;

  g_object_class_install_property(
    object_class,
    PROP_PREFERENCES,
    g_param_spec_object(
      "preferences",
      "Preferences",
      "Preferences to operate on",
      C4S_TYPE_PREFERENCES,
      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
    )
  );
  
  g_object_class_install_property(
    object_class,
    PROP_ENGINE_FUNCTION_LIST,
    g_param_spec_object(
      "engine-function-list",
      "Engine function list",
      "Engine function list that is refreshed when the user clicks the reload button",
      C4S_TYPE_ENGINE_FUNCTION_LIST,
      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
    )
  );
}

GType
c4s_preferences_docs_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sPreferencesDocsClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_preferences_docs_class_init,
      NULL,
      NULL,
      sizeof(C4sPreferencesDocs),
      0,
      (GInstanceInitFunc)c4s_preferences_docs_init,
      NULL
    };

    our_type = g_type_register_static(
      C4S_TYPE_PREFERENCES_PAGE,
      "C4sPreferencesDocs",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sPreferencesDocs*
c4s_preferences_docs_new(C4sPreferences* preferences,
                         C4sEngineFunctionList* engine_function_list)
{
  GObject* object;

  object = g_object_new(
    C4S_TYPE_PREFERENCES_DOCS,
    "preferences", preferences,
    "engine-function-list", engine_function_list,
    NULL
  );

  return C4S_PREFERENCES_DOCS(object);
}
