/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-preferencessettings.h>

#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtklabel.h>

#include <string.h>

typedef struct _C4sPreferencesSettingsPrivate C4sPreferencesSettingsPrivate;
struct _C4sPreferencesSettingsPrivate
{
  C4sPreferences* preferences;
  
  GtkWidget* tab_width;
  GtkWidget* auto_indentation;
  GtkWidget* highlight_brackets;
};

#define C4S_PREFERENCES_SETTINGS_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE( \
    (o), \
    C4S_TYPE_PREFERENCES_SETTINGS, \
    C4sPreferencesSettingsPrivate \
  ))

enum
{
  PROP_0,

  PROP_PREFERENCES
};

static C4sPreferencesPageClass* parent_class = NULL;

static void
c4s_preferences_settings_notify_cb(GObject* object,
                                   GParamSpec* pspec,
                                   gpointer user_data);

static void
c4s_preferences_settings_tab_width_value_changed_cb(G_GNUC_UNUSED
                                                    GtkWidget* widget,
                                                    gpointer user_data)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  settings = C4S_PREFERENCES_SETTINGS(user_data);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
    
    c4s_preferences_set_tab_width(
      priv->preferences,
      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(priv->tab_width))
    );
    
    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
  }
}

static void
c4s_preferences_settings_auto_indentation_toggled_cb(G_GNUC_UNUSED
                                                     GtkWidget* widget,
                                                     gpointer user_data)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  settings = C4S_PREFERENCES_SETTINGS(user_data);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
    
    c4s_preferences_set_auto_indentation(
      priv->preferences,
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->auto_indentation))
    );
    
    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
  }
}

static void
c4s_preferences_settings_highlight_brackets_toggled_cb(G_GNUC_UNUSED
                                                       GtkWidget* widget,
                                                       gpointer user_data)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  settings = C4S_PREFERENCES_SETTINGS(user_data);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
    
    c4s_preferences_set_highlight_brackets(
      priv->preferences,
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->highlight_brackets))
    );
    
    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );
  }
}

static void
c4s_preferences_settings_notify_cb(GObject* object,
                                   GParamSpec* pspec,
                                   gpointer user_data)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  settings = C4S_PREFERENCES_SETTINGS(user_data);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(strcmp(pspec->name, "tab-width") == 0)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->tab_width),
      G_CALLBACK(c4s_preferences_settings_tab_width_value_changed_cb),
      settings
    );
    
    gtk_spin_button_set_value(
      GTK_SPIN_BUTTON(priv->tab_width),
      c4s_preferences_get_tab_width(C4S_PREFERENCES(object))
    );
    
    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->tab_width),
      G_CALLBACK(c4s_preferences_settings_tab_width_value_changed_cb),
      settings
    );
  }
  else if(strcmp(pspec->name, "auto-indentation") == 0)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->auto_indentation),
      G_CALLBACK(c4s_preferences_settings_auto_indentation_toggled_cb),
      settings
    );

    gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(priv->auto_indentation),
      c4s_preferences_get_auto_indentation(C4S_PREFERENCES(object))
    );

    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->auto_indentation),
      G_CALLBACK(c4s_preferences_settings_auto_indentation_toggled_cb),
      settings
    );
  }
  else if(strcmp(pspec->name, "highlight-brackets") == 0)
  {
    g_signal_handlers_block_by_func(
      G_OBJECT(priv->auto_indentation),
      G_CALLBACK(c4s_preferences_settings_highlight_brackets_toggled_cb),
      settings
    );

    gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(priv->highlight_brackets),
      c4s_preferences_get_highlight_brackets(C4S_PREFERENCES(object))
    );

    g_signal_handlers_unblock_by_func(
      G_OBJECT(priv->highlight_brackets),
      G_CALLBACK(c4s_preferences_settings_highlight_brackets_toggled_cb),
      settings
    );
  }
}

static void
c4s_preferences_settings_set_preferences(C4sPreferencesSettings* settings,
                                         C4sPreferences* preferences)
{
  C4sPreferencesSettingsPrivate* priv;
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_disconnect_by_func(
      G_OBJECT(preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );

    g_object_unref(G_OBJECT(preferences));

    if(preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(settings), FALSE);
  }

  if(preferences != NULL)
  {
    g_signal_connect(
      G_OBJECT(preferences),
      "notify",
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );

    gtk_spin_button_set_value(
      GTK_SPIN_BUTTON(priv->tab_width),
      c4s_preferences_get_tab_width(preferences)
    );
    
    gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(priv->auto_indentation),
      c4s_preferences_get_auto_indentation(preferences)
    );
    
    gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(priv->highlight_brackets),
      c4s_preferences_get_highlight_brackets(preferences)
    );

    g_object_ref(G_OBJECT(preferences));

    if(priv->preferences == NULL)
      gtk_widget_set_sensitive(GTK_WIDGET(settings), TRUE);
  }

  priv->preferences = preferences;
}

static void
c4s_preferences_settings_init(C4sPreferencesSettings* settings)
{
  C4sPreferencesSettingsPrivate* priv;
  C4sOptionGroup* group;
  GtkWidget* hbox;
  GtkWidget* label;

  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  priv->preferences = NULL;
  
  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(settings),
    "Tab stops"
  );

  label = gtk_label_new("Tab width:");
  gtk_widget_show(label);

  priv->tab_width = gtk_spin_button_new_with_range(1.0, 8.0, 1.0);
  gtk_entry_set_activates_default(GTK_ENTRY(priv->tab_width), TRUE);
  gtk_spin_button_set_increments(GTK_SPIN_BUTTON(priv->tab_width), 1, 1);
  
  g_signal_connect(
    G_OBJECT(priv->tab_width),
    "value-changed",
    G_CALLBACK(c4s_preferences_settings_tab_width_value_changed_cb),
    settings
  );

  gtk_widget_show(priv->tab_width);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), priv->tab_width, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  gtk_box_pack_start(GTK_BOX(group->vbox), hbox, FALSE, FALSE, 0);

  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(settings),
    "Indentation"
  );
  
  priv->auto_indentation = gtk_check_button_new_with_label(
    "Enable automatic indentation"
  );
  
  g_signal_connect(
    G_OBJECT(priv->auto_indentation),
    "toggled",
    G_CALLBACK(c4s_preferences_settings_auto_indentation_toggled_cb),
    settings
  );

  gtk_widget_show(priv->auto_indentation);

  gtk_box_pack_start(
    GTK_BOX(group->vbox),
    priv->auto_indentation,
    FALSE,
    FALSE,
    0
  );
  
  group = c4s_preferences_page_add_option_group(
    C4S_PREFERENCES_PAGE(settings),
    "Bracket matching"
  );
  
  priv->highlight_brackets = gtk_check_button_new_with_label(
    "Highlight matching brackets"
  );
  
  g_signal_connect(
    G_OBJECT(priv->highlight_brackets),
    "toggled",
    G_CALLBACK(c4s_preferences_settings_highlight_brackets_toggled_cb),
    settings
  );

  gtk_widget_show(priv->highlight_brackets);

  gtk_box_pack_start(
    GTK_BOX(group->vbox),
    priv->highlight_brackets,
    FALSE,
    FALSE,
    0
  );
  
  gtk_widget_set_sensitive(GTK_WIDGET(settings), FALSE);
}

static void
c4s_preferences_settings_finalize(GObject* object)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;
  
  settings = C4S_PREFERENCES_SETTINGS(object);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  if(priv->preferences != NULL)
  {
    g_signal_handlers_disconnect_by_func(
      G_OBJECT(priv->preferences),
      G_CALLBACK(c4s_preferences_settings_notify_cb),
      settings
    );

    g_object_unref(priv->preferences);
  }

  G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
c4s_preferences_settings_set_property(GObject* object,
                                      guint prop_id,
                                      const GValue* value,
                                      GParamSpec* pspec)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_SETTINGS(object));

  settings = C4S_PREFERENCES_SETTINGS(object);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    c4s_preferences_settings_set_preferences(
      settings,
      C4S_PREFERENCES(g_value_get_object(value))
    );

    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_settings_get_property(GObject* object,
                                      guint prop_id,
                                      GValue* value,
                                      GParamSpec* pspec)
{
  C4sPreferencesSettings* settings;
  C4sPreferencesSettingsPrivate* priv;

  g_return_if_fail(C4S_IS_PREFERENCES_SETTINGS(object));

  settings = C4S_PREFERENCES_SETTINGS(object);
  priv = C4S_PREFERENCES_SETTINGS_PRIVATE(settings);

  switch(prop_id)
  {
  case PROP_PREFERENCES:
    g_value_set_object(value, priv->preferences);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    break;
  }
}

static void
c4s_preferences_settings_class_init(C4sPreferencesSettingsClass* klass)
{
  GObjectClass* object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);
  
  g_type_class_add_private(klass, sizeof(C4sPreferencesSettingsPrivate));

  object_class->finalize = c4s_preferences_settings_finalize;
  object_class->set_property = c4s_preferences_settings_set_property;
  object_class->get_property = c4s_preferences_settings_get_property;

  g_object_class_install_property(
    object_class,
    PROP_PREFERENCES,
    g_param_spec_object(
      "preferences",
      "Preferences",
      "Preferences to operate on",
      C4S_TYPE_PREFERENCES,
      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
    )
  );
}

GType
c4s_preferences_settings_get_type(void)
{
  static GType our_type = 0;

  if(our_type == 0)
  {
    static const GTypeInfo our_info = {
      sizeof(C4sPreferencesSettingsClass),
      (GBaseInitFunc)NULL,
      (GBaseFinalizeFunc)NULL,
      (GClassInitFunc)c4s_preferences_settings_class_init,
      NULL,
      NULL,
      sizeof(C4sPreferencesSettings),
      0,
      (GInstanceInitFunc)c4s_preferences_settings_init,
      NULL
    };

    our_type = g_type_register_static(
      C4S_TYPE_PREFERENCES_PAGE,
      "C4sPreferencesSettings",
      &our_info,
      0
    );
  }

  return our_type;
}

C4sPreferencesSettings*
c4s_preferences_settings_new(C4sPreferences* preferences)
{
  GObject* object;

  object = g_object_new(
    C4S_TYPE_PREFERENCES_SETTINGS,
    "preferences", preferences,
    NULL
  );

  return C4S_PREFERENCES_SETTINGS(object);
}
