/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __C4S_TCP_H__
#define __C4S_TCP_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define C4S_TYPE_TCP_CONNECTION			(c4s_tcp_connection_get_type())
#define C4S_TCP_CONNECTION(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), C4S_TYPE_TCP_CONNECTION, C4sTcpConnection))
#define C4S_TCP_CONNECTION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), C4S_TYPE_TCP_CONNECTION, C4sTcpConnectionClass))
#define C4S_IS_TCP_CONNECTION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), C4S_TYPE_TCP_CONNECTION))
#define C4S_IS_TCP_CONNECTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), C4S_TYPE_TCP_CONNECTION))
#define C4S_TCP_CONNECTION_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), C4S_TYPE_TCP_CONNECTION, C4sTcpConnectionClass))

typedef struct _C4sTcpConnectionClass C4sTcpConnectionClass;
typedef struct _C4sTcpConnection C4sTcpConnection;

typedef struct _C4sTcpConnector C4sTcpConnector;

typedef void(*C4sTcpResolvFunc)(gpointer);
typedef void(*C4sTcpConnectFunc)(C4sTcpConnection*, gpointer);
typedef void(*C4sTcpErrorFunc)(const GError*, gpointer);

typedef void(*C4sTcpIOFunc)(C4sTcpConnection*, GIOCondition, gpointer);

struct _C4sTcpConnectionClass
{
  GObjectClass parent_class;
};

struct _C4sTcpConnection
{
  GObject parent_instance;
};

/* Call this before making any other calls in this file */
gboolean
c4s_tcp_init(GError** error);

void
c4s_tcp_deinit(void);

C4sTcpConnector*
c4s_tcp_connect(const gchar* hostname,
                guint port,
                C4sTcpResolvFunc resolv_func,
                C4sTcpConnectFunc connect_func,
                C4sTcpErrorFunc error_func,
                gpointer user_data,
                GError** error);

/* Must not be called after either connect_func or error_func have
 * been called */
void
c4s_tcp_cancel(C4sTcpConnector* connector);

GType
c4s_tcp_connection_get_type(void) G_GNUC_CONST;

GIOStatus
c4s_tcp_connection_send(C4sTcpConnection* conn,
                        const void* buf,
                        gsize size,
                        gsize* bytes_written,
                        GError** error);

GIOStatus
c4s_tcp_connection_recv(C4sTcpConnection* conn,
                        void* buf,
                        gsize size,
                        gsize* bytes_read,
                        GError** error);

void
c4s_tcp_connection_watch(C4sTcpConnection* conn,
                         GIOCondition condition,
                         C4sTcpIOFunc io_func,
                         gpointer user_data);

G_END_DECLS

#endif /* __C4S_TCP_H__ */
