/* c4scripter - A GTK+ based C4Script editor
 * Copyright (C) 2006 Bernhard Bonigl, Armin Burgmeier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <c4scripter/c4s-textutil.h>

#include <gtk/gtkmain.h>
#include <glib/gunicode.h>

static gboolean c4s_is_identifier(gunichar c)
{
  return g_unichar_isalnum(c) || c == '_' || c == '#';
}

gboolean c4s_forward_spaces(GtkTextIter* iter)
{
  gboolean result;
  result = FALSE;

  while(g_unichar_isspace(gtk_text_iter_get_char(iter)))
  {
    gtk_text_iter_forward_char(iter);
    result = TRUE;
  }

  return result;
}

gboolean c4s_backward_spaces(GtkTextIter* iter)
{
  gboolean result;
  result = FALSE;

  while(gtk_text_iter_backward_char(iter) == TRUE)
  {
    if(!g_unichar_isspace(gtk_text_iter_get_char(iter)))
    {
      gtk_text_iter_forward_char(iter);
      break;
    }

    result = TRUE;
  }

  return result;
}

void c4s_forward_identifier(GtkTextIter* iter)
{
  while(c4s_is_identifier(gtk_text_iter_get_char(iter)) == TRUE)
    gtk_text_iter_forward_char(iter);
}

void c4s_backward_identifier(GtkTextIter* iter)
{
  while(gtk_text_iter_backward_char(iter) == TRUE)
  {
    if(!c4s_is_identifier(gtk_text_iter_get_char(iter)))
    {
      gtk_text_iter_forward_char(iter);
      break;
    }
  }
}

gboolean c4s_starts_identifier(const GtkTextIter* iter)
{
  GtkTextIter before;

  if(!c4s_is_identifier(gtk_text_iter_get_char(iter)))
    return FALSE;

  before = *iter;
  if(gtk_text_iter_backward_char(&before) == FALSE)
    return TRUE;

  return !c4s_is_identifier(gtk_text_iter_get_char(&before));
}

gboolean c4s_ends_identifier(const GtkTextIter* iter)
{
  GtkTextIter before;

  if(c4s_is_identifier(gtk_text_iter_get_char(iter)))
    return FALSE;

  before = *iter;
  if(gtk_text_iter_backward_char(&before) == FALSE)
    return FALSE;

  return c4s_is_identifier(gtk_text_iter_get_char(&before));
}

gboolean c4s_forward_search(const GtkTextIter* iter,
                            const gchar* str,
                            GtkTextSearchFlags flags,
                            GtkTextIter* match_start,
                            GtkTextIter* match_end,
                            const GtkTextIter* limit)
{
  const gboolean old_gtk = (
    gtk_major_version <= 2 &&
    gtk_minor_version <= 8 &&
    gtk_micro_version <= 6
  );

  GtkTextIter new_limit;

  if(old_gtk == TRUE)
  {
    if(gtk_text_iter_is_end(limit))
    {
      return gtk_text_iter_forward_search(
        iter,
        str,
        flags,
        match_start,
        match_end,
        NULL
      );
    }
    else
    {
      new_limit = *limit;
      gtk_text_iter_forward_char(&new_limit);

      return gtk_text_iter_forward_search(
        iter,
        str,
        flags,
        match_start,
        match_end,
        &new_limit
      );
    }
  }
  else
  {
    return gtk_text_iter_forward_search(
      iter,
      str,
      flags,
      match_start,
      match_end,
      limit
    );
  }
}

gboolean c4s_text_is_ascii(const gchar* text)
{
  for(; *text != '\0'; ++ text)
  {
    if((unsigned char)*text >= 0x80)
      return FALSE;
  }

  return TRUE;
}

gboolean c4s_substr_cmp(const gchar* str1,
                        gsize len1,
                        const gchar* str2,
                        gsize len2)
{
  if(len1 != len2) return FALSE;
  return strncmp(str1, str2, len1) == 0;
}
